/*********************************************************************
*	AFEDRI SDR-Net Commander utility
*	Author: Alexander Trushkin ( 4Z5LV )
*	Date: 9.10.2013
*	
*	This utility allows you to control AFEDRI SDR 
*	over USB connection.
*	So you can use this utility in command shell scripts or WebSDR
*	or in crone command   
**********************************************************************/

#include <stdio.h>
#ifdef __linux__
	#include <unistd.h>
#else
	#include "windows.h"
#endif
#include "hidapi.h"
#include "fe_interface.h"
#include "eeprom.h"
#include "commands.h"
#include "main.h"
#include "version.h"
#include "help.h"

//#define DEBUG 1

int RF_Gain_Db_to_Int(int value)
{
    int rf_gain=0;
//    rf_gain = RF_GAIN_ZERO + 3*(value>>3);
    rf_gain = ((value + 10) * 8) / 3;
    return rf_gain;	
}

int enumerate_afedri_devs()
{
	int res;
	unsigned char buf[1024];
	#define MAX_STR 255
	wchar_t wstr[1024];
	hid_device *handle;
	int i,k;

	// Enumerate and print the HID devices on the system
	struct hid_device_info *devs, *cur_dev;
	k = 0;
	printf("Trying to enumerate USB-HID devices ->");
	devs = hid_enumerate(HID_DEVICE_ATEECS_VID, HID_DEVICE_SDR_NET_PID);
	cur_dev = devs;	
	while (cur_dev) {
		printf("Device Found\n  type: %04hx %04hx\n  path: %s\n  serial_number: %ls",
			cur_dev->vendor_id, cur_dev->product_id, cur_dev->path, cur_dev->serial_number);
		printf("\n");
		printf("  Manufacturer: %ls\n", cur_dev->manufacturer_string);
		printf("  Product:      %ls\n", cur_dev->product_string);
		printf("\n");
		cur_dev = cur_dev->next;
		k++;
	}
	printf("PID(0x0002)=%i , ",k);
	hid_free_enumeration(devs);
	k = 0;
	devs = hid_enumerate(HID_DEVICE_ATEECS_VID, HID_DEVICE_ATEECS_PID_6);
	cur_dev = devs;	
	while (cur_dev) {
		printf("Device Found\n  type: %04hx %04hx\n  path: %s\n  serial_number: %ls",
			cur_dev->vendor_id, cur_dev->product_id, cur_dev->path, cur_dev->serial_number);
		printf("\n");
		printf("  Manufacturer: %ls\n", cur_dev->manufacturer_string);
		printf("  Product:      %ls\n", cur_dev->product_string);
		printf("\n");
		cur_dev = cur_dev->next;
		k++;
	}
	hid_free_enumeration(devs);
	printf("PID(0x0006)=%i\n\r",k);

}

unsigned long HID_wait_For_ACK()//int report_type, int command) 
{
	int res=0;
	unsigned long result=0;
	unsigned char buf[64]={0};
	unsigned char gen_command = 0;
#ifdef NON_BLOCKING_HID_IO
	while(res <= 0)
#endif
	{
//		Sleep(10);
		res = HID_Read_Data(buf);
		if(res > 0)
		{
			switch (buf[0])// == 6)
			{
				case 6:
					result = HID_Get_Generic_Report(&gen_command, (const unsigned char *)buf);
					if(gen_command == HID_GENERIC_ACK)
					{
#ifdef DEBUG					
						printf("ACK rprt=%i, cmd=%i \n\r",gen_command, result);
#endif
						return result;
					}
					else return result;
				case 7:
#ifdef DEBUG					
					printf("ACK Frequency \n\r");
#endif
					return 0;
			}
		}

#ifdef NON_BLOCKING_HID_IO
 #ifdef __linux__
		sleep(1);
 #else
		Sleep(1);
 #endif
#endif
//		else return;
	}


}

int initialize_hardware()
{
	char msg[128]={0};
	int res = 0;
	long result=0;
	unsigned char gen_command = 0;
	unsigned char buf[64]={0};
#ifdef NON_BLOCKING_HID_IO
	printf("Init HID interface - Non blocking I/O mode!!! \n");
#else
	printf("Init HID interface - Blocking I/O mode!!! \n");
#endif
	res = HID_Device_Init(msg);
	if(!res)
	{
		printf("%s",msg);
		return -1;
	}
	else
	{
		printf("%s device was found!!!\n", msg);
		return 0;
	}
}

int set_rf_gain(int gain)
{
	int res;
	res = RF_Gain_Db_to_Int(gain);
	HID_ChangeRF_Gain(0,res);
	HID_wait_For_ACK();
	printf("RF gain %idB => %i\r\n", gain, res);
	return 0;
}

int set_rf_gain_indexed(int gain)
{
	int res;
	res = gain << 3;
	HID_ChangeRF_Gain(0,res);
	HID_wait_For_ACK();
	printf("RF gain index => %i\r\n", res);
	return 0;
}

int save_defaults()
{
	return HID_Send_defaults_to_EEPROM();
}

int set_fe_gain(int gain)
{
	if(gain >= 0 && gain < 7)
	{
		HID_ChangeFE_Gain(GainIndex[gain]);
		HID_wait_For_ACK();
		printf("FE gain %f\r\n", GainTable[gain]);
	}
	else
	{
		printf("Error - FE gain out of range! \n\r");
		printf("FE Gain should be: 0 to 6 \n\r");
		return -1;
	}
	return 0;
}

int set_rx_frequency(unsigned long frequency)
{
	HID_Set_Frequency(frequency);
	HID_wait_For_ACK();
	printf("RX Frequency %u (Hz)\r\n", frequency);
	return 0;
}

int set_usb_mode(unsigned long mode)
{
	HID_Send_USB_Mode(mode);
	HID_wait_For_ACK();
	if(mode) printf("The new mode is USB Audio...\r\n");
	else printf("The new mode is USB MSD ...\r\n");
	return 0;
}

int save_main_clock_freq(unsigned long frequency)
{
    int  res;
    unsigned long tmp=0;
    tmp = frequency & 0xFFFF;
    res = HID_Send_EEPROM_Data(VADDRESS_MAIN_CLOCK_FREQ_LOW_HALFWORD, tmp);
    tmp = (frequency >> 16) & 0xFFFF;
	HID_wait_For_ACK();
    res = HID_Send_EEPROM_Data(VADDRESS_MAIN_CLOCK_FREQ_HIGH_HALFWORD, tmp);
	HID_wait_For_ACK();
	printf("The new Main Clock frequency was saved:  %u (Hz)\r\n", frequency);
	return 0;
}

int save_usb_sample_rate(unsigned long SampleRate)
{
        int  res;
        unsigned long tmp=0;
        tmp = SampleRate & 0xFFFF;
        res = HID_Send_EEPROM_Data(VADDRESS_SAMPLE_RATE_LO, tmp);
        tmp = (SampleRate >> 16) & 0xFFFF;
	HID_wait_For_ACK();
        res = HID_Send_EEPROM_Data(VADDRESS_SAMPLE_RATE_HI, tmp);
	HID_wait_For_ACK();
	printf("The new USB sample rate was saved:  %u (Hz)\r\n", SampleRate);
	return 0;
}

int Get_SDR_Information()
{
 unsigned long result = 0;
 unsigned long main_clock_freq = 0;
        HID_Check_FW_Version(0);
	result = HID_wait_For_ACK();
	printf("Firmware version: %X\n\r", result);
        HID_Get_Serial_Number();
	result  = HID_wait_For_ACK();
	printf("SDR serial number: %X\n\r", result);
        HID_Get_HW_CPLD_SN();
	result = HID_wait_For_ACK();
	printf("CPLD serial number: %X\n\r", result);
        HID_Get_EEPROM_Data(VADDRESS_MAIN_CLOCK_FREQ_LOW_HALFWORD);
	result = HID_wait_For_ACK();
	main_clock_freq += result & 0xFFFF;
        HID_Get_EEPROM_Data(VADDRESS_MAIN_CLOCK_FREQ_HIGH_HALFWORD);
	result = HID_wait_For_ACK();
	main_clock_freq += (result  << 16) & 0xFFFF0000;
	printf("Main clock frequency: %u\n\r", main_clock_freq);
	HID_Get_Sample_Rate();
	result = HID_wait_For_ACK();
	printf("USB Sample Rate: %u\n\r", result);
        HID_Get_SDR_IP_Address();
	result = HID_wait_For_ACK();
	printf("SDR IP address: %u.%u.%u.%u\n\r", (result >> 24) & 0xFF, (result >> 16) & 0xFF, (result >> 8) & 0xFF, result & 0xFF);
        HID_Get_SDR_IP_Mask();
	result = HID_wait_For_ACK();
	printf("SDR IP mask: %u.%u.%u.%u\n\r", (result >> 24) & 0xFF, (result >> 16) & 0xFF, (result >> 8) & 0xFF, result & 0xFF);
        HID_Get_Gateway_IP_Address();
	result = HID_wait_For_ACK();
	printf("Gateway IP address: %u.%u.%u.%u\n\r", (result >> 24) & 0xFF, (result >> 16) & 0xFF, (result >> 8) & 0xFF, result & 0xFF);
        HID_Get_UDP_Port();
	result = HID_wait_For_ACK();
	printf("UDP port number: %u\n\r", result);
        HID_Get_DHCP_Server_State();
	result = HID_wait_For_ACK();
	printf("DHCP server state: %X\n\r", result);
	return 0;
}

int reset_sdr()
{
	HID_SoftReset_SDR();
	HID_Close();
 #ifdef __linux__
		sleep(1);
 #else
		Sleep(1);
 #endif
	printf("Reset SDR!\n\r");
	return 0;
}

int show_help_information()
{
	printf("SDR Commander utility version %s\n\r", APP_VERSION);
	printf("%s\n\r", help_text);
	return 0;
}
//End of File

